@@ -18,21 +18,28 @@
 
 #include <QTabBar>
 #include <QInputDialog>
+#include <QColorDialog>
 #include <QMouseEvent>
 #include <QMenu>
 
+#include "mainwindow.h"
 #include "termwidgetholder.h"
+#include "tabbar.h"
 #include "tabwidget.h"
 #include "config.h"
 #include "properties.h"
+#include "qterminalapp.h"
 
 
 #define TAB_INDEX_PROPERTY "tab_index"
 #define TAB_CUSTOM_NAME_PROPERTY "custom_name"
 
 
-TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent), tabNumerator(0)
+TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent), tabNumerator(0), mTabBar(new TabBar(this))
 {
+    // Insert our own tab bar which overrides tab width and eliding
+    setTabBar(mTabBar);
+
     setFocusPolicy(Qt::NoFocus);
 
     /* On Mac OS X this will look similar to
@@ -43,15 +50,15 @@ TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent), tabNumerator(0)
 
     tabBar()->setUsesScrollButtons(true);
 
-    setTabsClosable(true);
     setMovable(true);
     setUsesScrollButtons(true);
 
     tabBar()->installEventFilter(this);
 
-    connect(this, SIGNAL(tabCloseRequested(int)), this, SLOT(removeTab(int)));
-    connect(tabBar(), SIGNAL(tabMoved(int,int)), this, SLOT(updateTabIndices()));
-    connect(this, SIGNAL(tabRenameRequested(int)), this, SLOT(renameSession(int)));
+    connect(this, &TabWidget::tabCloseRequested, this, &TabWidget::removeTab);
+    connect(tabBar(), &QTabBar::tabMoved, this, &TabWidget::updateTabIndices);
+    connect(this, &TabWidget::tabRenameRequested, this, &TabWidget::renameSession);
+    connect(this, &TabWidget::tabTitleColorChangeRequested, this, &TabWidget::setTitleColor);
 }
 
 TermWidgetHolder * TabWidget::terminalHolder()
@@ -59,29 +66,20 @@ TermWidgetHolder * TabWidget::terminalHolder()
     return reinterpret_cast<TermWidgetHolder*>(widget(currentIndex()));
 }
 
-void TabWidget::setWorkDirectory(const QString& dir)
-{
-    this->work_dir = dir;
-}
 
-int TabWidget::addNewTab(const QString & shell_program)
+int TabWidget::addNewTab(TerminalConfig config)
 {
     tabNumerator++;
     QString label = QString(tr("Shell No. %1")).arg(tabNumerator);
 
     TermWidgetHolder *ch = terminalHolder();
-    QString cwd(work_dir);
-    if (Properties::Instance()->useCWD && ch)
-    {
-        cwd = ch->currentTerminal()->impl()->workingDirectory();
-        if (cwd.isEmpty())
-            cwd = work_dir;
-    }
+    if (ch)
+        config.provideCurrentDirectory(ch->currentTerminal()->impl()->workingDirectory());
 
-    TermWidgetHolder *console = new TermWidgetHolder(cwd, shell_program, this);
+    TermWidgetHolder *console = new TermWidgetHolder(config, this);
     console->setWindowTitle(label);
-    connect(console, SIGNAL(finished()), SLOT(removeFinished()));
-    connect(console, SIGNAL(lastTerminalClosed()), this, SLOT(removeFinished()));
+    connect(console, &TermWidgetHolder::finished, this, &TabWidget::removeFinished);
+    connect(console, &TermWidgetHolder::lastTerminalClosed, this, &TabWidget::removeFinished);
     connect(console, &TermWidgetHolder::termTitleChanged, this, &TabWidget::onTermTitleChanged);
     connect(this, &QTabWidget::currentChanged, this, &TabWidget::currentTitleChanged);
 
@@ -96,29 +94,40 @@ int TabWidget::addNewTab(const QString & shell_program)
     return index;
 }
 
-void TabWidget::switchNextSubterminal()
+void TabWidget::switchLeftSubterminal()
 {
-    terminalHolder()->switchNextSubterminal();
+    terminalHolder()->directionalNavigation(NavigationDirection::Left);
 }
 
-void TabWidget::switchPrevSubterminal()
+void TabWidget::switchRightSubterminal()
 {
-    terminalHolder()->switchPrevSubterminal();
+    terminalHolder()->directionalNavigation(NavigationDirection::Right);
+}
+
+void TabWidget::switchTopSubterminal() {
+    terminalHolder()->directionalNavigation(NavigationDirection::Top);
+}
+
+void TabWidget::switchBottomSubterminal() {
+    terminalHolder()->directionalNavigation(NavigationDirection::Bottom);
 }
 
 void TabWidget::splitHorizontally()
 {
     terminalHolder()->splitHorizontal(terminalHolder()->currentTerminal());
+    findParent<MainWindow>(this)->updateDisabledActions();
 }
 
 void TabWidget::splitVertically()
 {
     terminalHolder()->splitVertical(terminalHolder()->currentTerminal());
+    findParent<MainWindow>(this)->updateDisabledActions();
 }
 
 void TabWidget::splitCollapse()
 {
     terminalHolder()->splitCollapse(terminalHolder()->currentTerminal());
+    findParent<MainWindow>(this)->updateDisabledActions();
 }
 
 void TabWidget::copySelection()
@@ -193,6 +202,15 @@ void TabWidget::renameCurrentSession()
     renameSession(currentIndex());
 }
 
+void TabWidget::setTitleColor(int index)
+{
+    QColor current = tabBar()->tabTextColor(index);
+    QColor color = QColorDialog::getColor(current, this, tr("Select new tab title color"));
+
+    if (color.isValid())
+        tabBar()->setTabTextColor(index, color);
+}
+
 void TabWidget::renameTabsAfterRemove()
 {
 // it breaks custom names - it replaces original/custom title with shell no #
@@ -206,31 +224,48 @@ void TabWidget::renameTabsAfterRemove()
 void TabWidget::contextMenuEvent(QContextMenuEvent *event)
 {
     QMenu menu(this);
+    QMap< QString, QAction * > actions = findParent<MainWindow>(this)->leaseActions();
 
     QAction *close = menu.addAction(QIcon::fromTheme("document-close"), tr("Close session"));
-    QAction *rename = menu.addAction(Properties::Instance()->actions[RENAME_SESSION]->text());
-    rename->setShortcut(Properties::Instance()->actions[RENAME_SESSION]->shortcut());
+    QAction *rename = menu.addAction(actions[RENAME_SESSION]->text());
+    QAction *changeColor = menu.addAction(QIcon::fromTheme("color-management"), tr("Change title color"));
+    rename->setShortcut(actions[RENAME_SESSION]->shortcut());
     rename->blockSignals(true);
 
-    int tabIndex = tabBar()->tabAt(event->pos());
+    int tabIndex = tabBar()->tabAt(tabBar()->mapFrom(this,event->pos()));
     QAction *action = menu.exec(event->globalPos());
     if (action == close) {
         emit tabCloseRequested(tabIndex);
     } else if (action == rename) {
         emit tabRenameRequested(tabIndex);
+    } else if (action == changeColor) {
+	emit tabTitleColorChangeRequested(tabIndex);
     }
 }
 
 bool TabWidget::eventFilter(QObject *obj, QEvent *event)
 {
-    if (event->type() == QEvent::MouseButtonDblClick)
+    QMouseEvent *e = reinterpret_cast<QMouseEvent*>(event);
+    if (e->button() == Qt::MidButton) {
+        if (event->type() == QEvent::MouseButtonRelease) {
+            // close the tab on middle clicking
+            int index = tabBar()->tabAt(e->pos());
+            if (index > -1){
+                removeTab(index);
+                return true;
+            }
+        }
+    }
+    else if (event->type() == QEvent::MouseButtonDblClick)
     {
-        QMouseEvent *e = reinterpret_cast<QMouseEvent*>(event);
         // if user doubleclicks on tab button - rename it. If user
         // clicks on free space - open new tab
         int index = tabBar()->tabAt(e->pos());
         if (index == -1)
-            addNewTab();
+        {
+            TerminalConfig defaultConfig;
+            addNewTab(defaultConfig);
+        }
         else
             renameSession(index);
         return true;
@@ -270,7 +305,7 @@ void TabWidget::removeTab(int index)
     //    tabNumerator--;
         setUpdatesEnabled(true);
     } else {
-        emit closeTabNotification();
+        emit closeTabNotification(true);
     }
 
     renameTabsAfterRemove();
@@ -288,7 +323,7 @@ void TabWidget::removeCurrentTab()
     if (count() > 1) {
         removeTab(currentIndex());
     } else {
-        emit closeTabNotification();
+        emit closeTabNotification(false);
     }
 }
 
@@ -299,6 +334,7 @@ int TabWidget::switchToRight()
         setCurrentIndex(next_pos);
     else
         setCurrentIndex(0);
+    findParent<MainWindow>(this)->updateDisabledActions();
     return currentIndex();
 }
 
@@ -309,6 +345,7 @@ int TabWidget::switchToLeft()
         setCurrentIndex(count() - 1);
     else
         setCurrentIndex(previous_pos);
+    findParent<MainWindow>(this)->updateDisabledActions();
     return currentIndex();
 }
 
@@ -406,6 +443,13 @@ void TabWidget::propertiesChanged()
         console->propertiesChanged();
     }
     showHideTabBar();
+
+    setTabsClosable(Properties::Instance()->showCloseTabButton);
+
+    // Update the tab widths
+    mTabBar->setLimitWidth(Properties::Instance()->limitTabWidth);
+    mTabBar->setLimitWidthValue(Properties::Instance()->limitTabWidthValue);
+    mTabBar->updateWidth();
 }
 
 void TabWidget::clearActiveTerminal()
@@ -426,33 +470,36 @@ void TabWidget::loadSession()
 
 void TabWidget::preset2Horizontal()
 {
-    int ix = TabWidget::addNewTab();
+    TerminalConfig defaultConfig;
+    int ix = TabWidget::addNewTab(defaultConfig);
     TermWidgetHolder* term = reinterpret_cast<TermWidgetHolder*>(widget(ix));
     term->splitHorizontal(term->currentTerminal());
     // switch to the 1st terminal
-    term->switchNextSubterminal();
+    term->directionalNavigation(NavigationDirection::Left);
 }
 
 void TabWidget::preset2Vertical()
 {
-    int ix = TabWidget::addNewTab();
+    TerminalConfig defaultConfig;
+    int ix = TabWidget::addNewTab(defaultConfig);
     TermWidgetHolder* term = reinterpret_cast<TermWidgetHolder*>(widget(ix));
     term->splitVertical(term->currentTerminal());
     // switch to the 1st terminal
-    term->switchNextSubterminal();
+    term->directionalNavigation(NavigationDirection::Left);
 }
 
 void TabWidget::preset4Terminals()
 {
-    int ix = TabWidget::addNewTab();
+    TerminalConfig defaultConfig;
+    int ix = TabWidget::addNewTab(defaultConfig);
     TermWidgetHolder* term = reinterpret_cast<TermWidgetHolder*>(widget(ix));
     term->splitVertical(term->currentTerminal());
     term->splitHorizontal(term->currentTerminal());
-    term->switchNextSubterminal();
-    term->switchNextSubterminal();
+    term->directionalNavigation(NavigationDirection::Left);
+
     term->splitHorizontal(term->currentTerminal());
     // switch to the 1st terminal
-    term->switchNextSubterminal();
+    term->directionalNavigation(NavigationDirection::Top);
 }
 
 void TabWidget::showHideTabBar()
